unit XYZForm;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  ShellAPI, Menus, StdCtrls, IdBaseComponent, IdComponent, IdTCPServer,
  ComCtrls;

const
  WM_XYZTRAYICON = WM_USER + 1111;
  WM_XYZTRAYICON_SHOW = WM_XYZTRAYICON + 1;
  XYZ_INITIALIZE = WM_XYZTRAYICON + 2;

type
  TXYZMainForm = class(TForm)
    XYZTryIconPopupMenu: TPopupMenu;
    XYZTryIconPopUpMenuInfo: TMenuItem;
    XYZTryIconPopUpMenuShow: TMenuItem;
    XYZTryIconPopUpMenuClose: TMenuItem;
    XYZMessageMemo: TMemo;
    IdTCPServer1: TIdTCPServer;
    StatusBar1: TStatusBar;
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure XYZTryIconPopUpMenuCloseClick(Sender: TObject);
    procedure XYZTryIconPopUpMenuInfoClick(Sender: TObject);
    procedure XYZTryIconPopUpMenuShowClick(Sender: TObject);
    procedure IdTCPServer1Connect(AThread: TIdPeerThread);
    procedure IdTCPServer1Disconnect(AThread: TIdPeerThread);
    procedure IdTCPServer1Execute(AThread: TIdPeerThread);
  private
    { Private declarations }
    FXYZTryIconAlive: Boolean;
    FXYZTryIconData: TNotifyIconData;
    FTaskBarCreateMessage: DWord;
    FFromServcice: Boolean;
    FCaptionString: String;
    FClientCount: Integer;
  protected
    { Protected declarations }
    procedure WndProc(var Message: TMessage); override;
    procedure AddIcon;
    procedure WMXYZTRrayIconShow(var Message: TMessage); message WM_XYZTRAYICON_SHOW;
    procedure WMXYZTRrayIcon(var Message: TMessage); message WM_XYZTRAYICON;
    procedure XYZInitialize(var Message: TMessage); message XYZ_INITIALIZE;
  public
    { Public declarations }
    Procedure Initialize(FromServcice: Boolean);
  end;

var
  XYZMainForm: TXYZMainForm;

implementation

{$R *.DFM}

procedure TXYZMainForm.FormCreate(Sender: TObject);
begin
  FTaskBarCreateMessage:= RegisterWindowMessage('TaskbarCreated');
  FClientCount:= 0;
  FXYZTryIconAlive:= FALSE;
  FFromServcice:= FALSE;
  FCaptionString:= Caption;
end;

procedure TXYZMainForm.FormDestroy(Sender: TObject);
begin
  If FXYZTryIconAlive then begin
    FXYZTryIconAlive:= Shell_NotifyIcon(NIM_DELETE, @FXYZTryIconData);
  end;
  If IdTCPServer1.Active Then
    IdTCPServer1.Active:= False;
end;

procedure TXYZMainForm.WMXYZTRrayIconShow(var Message: TMessage);
begin
  Sleep( 0 );
  AddIcon;
end;

procedure TXYZMainForm.WndProc(var Message: TMessage);
begin
  if Message.Msg = FTaskBarCreateMessage then begin
    Message.Result:= 0;
    PostMessage( Handle, WM_XYZTRAYICON_SHOW, 0, 0 );
  end else
    inherited WndProc(Message);
end;

procedure TXYZMainForm.AddIcon;
begin
  If FXYZTryIconAlive then begin
    FXYZTryIconAlive:= Shell_NotifyIcon(NIM_DELETE, @FXYZTryIconData);
  end;
  FXYZTryIconData.cbSize:= SizeOf(FXYZTryIconData);
  FXYZTryIconData.Wnd:= Self.Handle;
  FXYZTryIconData.uID:= $DEDB;
  FXYZTryIconData.uFlags:= NIF_MESSAGE or NIF_ICON or NIF_TIP;
  FXYZTryIconData.hIcon:= Forms.Application.Icon.Handle;
  FXYZTryIconData.uCallbackMessage:= WM_XYZTRAYICON;
  StrCopy(FXYZTryIconData.szTip, PChar(Caption));
  FXYZTryIconAlive:= Shell_NotifyIcon(NIM_Add, @FXYZTryIconData);
end;

procedure TXYZMainForm.Initialize( FromServcice: Boolean );
begin
  FFromServcice:= FromServcice;
  XYZMessageMemo.Lines.Clear;
  If FFromServcice Then Begin
    XYZTryIconPopUpMenuClose.Visible:= False;
    XYZMessageMemo.Lines.Add('Satrted as a "Service"');
  end else begin
    XYZMessageMemo.Lines.Add('Satrted as an "Application"');
  end;
  StatusBar1.SimpleText:= FCaptionString;
  If NOT IdTCPServer1.Active Then
    IdTCPServer1.Active:= True;
  AddIcon;
end;

procedure TXYZMainForm.XYZInitialize(var Message: TMessage);
begin
  Initialize(Message.WParam <> 0);
end;

procedure TXYZMainForm.WMXYZTRrayIcon(var Message: TMessage);
var
  pt: TPoint;
begin
  case Message.LParam of
    WM_RBUTTONUP: begin
      if Visible then begin
        SetForegroundWindow(Handle);
      end else begin
        SetForegroundWindow(Handle);
        GetCursorPos(pt);
        XYZTryIconPopupMenu.Popup(pt.x, pt.y);
      end;
    end;
    WM_LBUTTONDBLCLK: begin
      if Visible then
        SetForegroundWindow(Handle)
      else
        ShowModal;
    end;
  end;
end;

procedure TXYZMainForm.XYZTryIconPopUpMenuInfoClick(Sender: TObject);
begin
  MessageBox(0, '(0.0a) Version: 0 Revision: 0 Built: alpha', 'XYZ Service with TryIcon', MB_ICONHAND);
end;

procedure TXYZMainForm.XYZTryIconPopUpMenuShowClick(Sender: TObject);
begin
  ShowModal;
end;

procedure TXYZMainForm.XYZTryIconPopUpMenuCloseClick(Sender: TObject);
begin
  Close;
end;

procedure TXYZMainForm.IdTCPServer1Connect(AThread: TIdPeerThread);
begin
  Inc( FClientCount );
  StatusBar1.SimpleText:= FCaptionString+' '+IntToStr( FClientCount );
end;

procedure TXYZMainForm.IdTCPServer1Disconnect(AThread: TIdPeerThread);
begin
  Dec( FClientCount );
  If FClientCount=0 Then
    StatusBar1.SimpleText:= FCaptionString
  Else
    StatusBar1.SimpleText:= FCaptionString+' '+IntToStr( FClientCount );
end;

procedure TXYZMainForm.IdTCPServer1Execute(AThread: TIdPeerThread);
var
  S: String;
begin
  S:= AThread.Connection.ReadLn;
  If Length(S) > 0 Then
    XYZMessageMemo.Lines.Add( S );
  AThread.Connection.WriteLn( '->['+S+']' );
  If AnsiSameText('QUIT', S ) or AnsiSameText('EXIT', S ) Then
    AThread.Connection.Disconnect;
end;

end.
